"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.webPerformanceApi = void 0;

var _uniqueId = _interopRequireDefault(require("lodash/uniqueId"));

/**
 * Measure performances since the given markName
 *
 * @param {import('./types').MeasureOptions} options - Options for the measurement
 */
var measure = function measure(_ref) {
  var measureName = _ref.measureName,
      markName = _ref.markName,
      category = _ref.category,
      color = _ref.color;
  var name = measureName !== null && measureName !== void 0 ? measureName : markName; // @ts-ignore

  performance.measure(name, {
    start: markName,
    detail: {
      devtools: {
        track: category !== null && category !== void 0 ? category : 'CozyClient',
        color: color !== null && color !== void 0 ? color : 'secondary'
      }
    }
  });
};
/**
 * Set a mark with the given name
 *
 * In order to ensure uniqueness of mark names,
 * the given markName is prefixed with a unique id
 *
 * This method returns the computed mark name containing
 * the unique id. This new ID should be used when
 * calling `measure()`
 *
 * @param {string} markName - the name of the mark
 */


var mark = function mark(markName) {
  var uniqMarkName = "".concat((0, _uniqueId.default)(), " ").concat(markName);
  performance.mark(uniqMarkName);
  return uniqMarkName;
};
/** @type {import('./types').PerformanceAPI} */


var webPerformanceApi = {
  measure: measure,
  mark: mark
};
exports.webPerformanceApi = webPerformanceApi;