"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.persistVirtualDocuments = exports.shouldDocumentBePersisted = exports.shouldEnforcePersist = exports.ENFORCE_LIST = void 0;

var _regenerator = _interopRequireDefault(require("@babel/runtime/regenerator"));

var _asyncToGenerator2 = _interopRequireDefault(require("@babel/runtime/helpers/asyncToGenerator"));

var _CozyClient = _interopRequireDefault(require("./CozyClient"));

var _dsl = require("./queries/dsl");

function _createForOfIteratorHelper(o, allowArrayLike) { var it; if (typeof Symbol === "undefined" || o[Symbol.iterator] == null) { if (Array.isArray(o) || (it = _unsupportedIterableToArray(o)) || allowArrayLike && o && typeof o.length === "number") { if (it) o = it; var i = 0; var F = function F() {}; return { s: F, n: function n() { if (i >= o.length) return { done: true }; return { done: false, value: o[i++] }; }, e: function e(_e) { throw _e; }, f: F }; } throw new TypeError("Invalid attempt to iterate non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method."); } var normalCompletion = true, didErr = false, err; return { s: function s() { it = o[Symbol.iterator](); }, n: function n() { var step = it.next(); normalCompletion = step.done; return step; }, e: function e(_e2) { didErr = true; err = _e2; }, f: function f() { try { if (!normalCompletion && it.return != null) it.return(); } finally { if (didErr) throw err; } } }; }

function _unsupportedIterableToArray(o, minLen) { if (!o) return; if (typeof o === "string") return _arrayLikeToArray(o, minLen); var n = Object.prototype.toString.call(o).slice(8, -1); if (n === "Object" && o.constructor) n = o.constructor.name; if (n === "Map" || n === "Set") return Array.from(o); if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray(o, minLen); }

function _arrayLikeToArray(arr, len) { if (len == null || len > arr.length) len = arr.length; for (var i = 0, arr2 = new Array(len); i < len; i++) { arr2[i] = arr[i]; } return arr2; }

var ENFORCE_LIST = ['io.cozy.files.shortcuts'];
/**
 * @typedef {import("./types").CozyClientDocument} CozyClientDocument
 */

exports.ENFORCE_LIST = ENFORCE_LIST;

var shouldEnforcePersist = function shouldEnforcePersist(definition) {
  return ENFORCE_LIST.includes(definition.doctype);
};
/**
 * Whether or not a document should be locally persisted
 *
 * @param {CozyClientDocument} document - The document to evaluate
 * @returns {boolean} whether or not the doc should be persisted
 */


exports.shouldEnforcePersist = shouldEnforcePersist;

var shouldDocumentBePersisted = function shouldDocumentBePersisted(document) {
  var _document$meta;

  var shouldEnforce = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : false;

  if (!document || document.cozyLocalOnly) {
    return false;
  }

  if (!((_document$meta = document.meta) !== null && _document$meta !== void 0 && _document$meta.rev) && !document._rev || shouldEnforce) {
    return true;
  }

  return false;
};
/**
 * Save the document or array of documents into the persisted storage (if any)
 *
 * @param {CozyClient} client - Cozy client instance
 * @param {QueryDefinition} definition - The query definition
 * @param {CozyClientDocument | Array<CozyClientDocument>} data - Document or array of documents to be saved
 * @returns {Promise<void>}
 */


exports.shouldDocumentBePersisted = shouldDocumentBePersisted;

var persistVirtualDocuments = /*#__PURE__*/function () {
  var _ref = (0, _asyncToGenerator2.default)( /*#__PURE__*/_regenerator.default.mark(function _callee(client, definition, data) {
    var enforce, documentsToPersist, _iterator, _step, document;

    return _regenerator.default.wrap(function _callee$(_context) {
      while (1) {
        switch (_context.prev = _context.next) {
          case 0:
            enforce = shouldEnforcePersist(definition);

            if (!(definition.doctype === 'io.cozy.apps_registry')) {
              _context.next = 5;
              break;
            }

            _context.next = 4;
            return persistVirtualDocument(client, {
              _type: 'io.cozy.apps_registry',
              _id: definition.id,
              // @ts-ignore
              cozyPouchData: data
            }, enforce);

          case 4:
            return _context.abrupt("return", _context.sent);

          case 5:
            if (Array.isArray(data)) {
              _context.next = 10;
              break;
            }

            _context.next = 8;
            return persistVirtualDocument(client, data, enforce);

          case 8:
            _context.next = 28;
            break;

          case 10:
            documentsToPersist = data.filter(function (document) {
              return shouldDocumentBePersisted(document, enforce);
            });
            _iterator = _createForOfIteratorHelper(documentsToPersist);
            _context.prev = 12;

            _iterator.s();

          case 14:
            if ((_step = _iterator.n()).done) {
              _context.next = 20;
              break;
            }

            document = _step.value;
            _context.next = 18;
            return persistVirtualDocument(client, document, enforce);

          case 18:
            _context.next = 14;
            break;

          case 20:
            _context.next = 25;
            break;

          case 22:
            _context.prev = 22;
            _context.t0 = _context["catch"](12);

            _iterator.e(_context.t0);

          case 25:
            _context.prev = 25;

            _iterator.f();

            return _context.finish(25);

          case 28:
          case "end":
            return _context.stop();
        }
      }
    }, _callee, null, [[12, 22, 25, 28]]);
  }));

  return function persistVirtualDocuments(_x, _x2, _x3) {
    return _ref.apply(this, arguments);
  };
}();
/**
 * Save the document or array of documents into the persisted storage (if any)
 *
 * This relies on the fact that there are some "virtual" documents returned
 * by the stack, in the sense that they are not CouchDB documents, but rather
 * are computed on the fly by the server.
 * To ensure we can use those documents locally, i.e. without the stack,
 * we allow the possibility to persist them.
 *
 * @param {CozyClient} client - Cozy client instance
 * @param {CozyClientDocument} document - Document to be saved
 * @param {boolean} enforce - When true, save the document no matter its content
 * @returns {Promise<void>}
 */


exports.persistVirtualDocuments = persistVirtualDocuments;

var persistVirtualDocument = /*#__PURE__*/function () {
  var _ref2 = (0, _asyncToGenerator2.default)( /*#__PURE__*/_regenerator.default.mark(function _callee2(client, document, enforce) {
    return _regenerator.default.wrap(function _callee2$(_context2) {
      while (1) {
        switch (_context2.prev = _context2.next) {
          case 0:
            if (!shouldDocumentBePersisted(document, enforce)) {
              _context2.next = 3;
              break;
            }

            _context2.next = 3;
            return client.chain.persistCozyData(document);

          case 3:
          case "end":
            return _context2.stop();
        }
      }
    }, _callee2);
  }));

  return function persistVirtualDocument(_x4, _x5, _x6) {
    return _ref2.apply(this, arguments);
  };
}();