"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.extractAndMergeDocument = exports.getCollectionFromSlice = exports.getDocumentsFromSlice = exports.getDocumentFromSlice = exports.default = exports.mergeDocumentsWithRelationships = void 0;

var _defineProperty2 = _interopRequireDefault(require("@babel/runtime/helpers/defineProperty"));

var _get = _interopRequireDefault(require("lodash/get"));

var _omit = _interopRequireDefault(require("lodash/omit"));

var _fastDeepEqual = _interopRequireDefault(require("fast-deep-equal"));

var _deepmerge = _interopRequireDefault(require("@fastify/deepmerge"));

var _logger = _interopRequireDefault(require("../logger"));

var _queries = require("./queries");

var _dsl = require("../queries/dsl");

var _mutations = require("./mutations");

var _helpers = require("./helpers");

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { (0, _defineProperty2.default)(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

var deepmergeFn = (0, _deepmerge.default)();

var storeDocument = function storeDocument(state, document) {
  var type = document._type;

  if (!type) {
    if (process.env.NODE_ENV !== 'production') {
      _logger.default.info('Document without _type', document);
    }

    throw new Error('Document without _type');
  }

  if (!(0, _helpers.properId)(document)) {
    if (process.env.NODE_ENV !== 'production') {
      _logger.default.info('Document without id', document);
    }

    throw new Error('Document without id');
  }

  var existingDoc = (0, _get.default)(state, [type, (0, _helpers.properId)(document)]);

  if ((0, _fastDeepEqual.default)(existingDoc, document)) {
    return state;
  } else {
    return _objectSpread(_objectSpread({}, state), {}, (0, _defineProperty2.default)({}, type, _objectSpread(_objectSpread({}, state[type]), {}, (0, _defineProperty2.default)({}, (0, _helpers.properId)(document), mergeDocumentsWithRelationships(existingDoc, document)))));
  }
};

var mergeDocumentsWithRelationships = function mergeDocumentsWithRelationships() {
  var prevDocument = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
  var nextDocument = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};

  /**
   * @type {import("../types").CozyClientDocument}
   */
  var merged = _objectSpread(_objectSpread({}, prevDocument), nextDocument);

  if (prevDocument.relationships || nextDocument.relationships) merged.relationships = _objectSpread(_objectSpread({}, prevDocument.relationships), nextDocument.relationships);
  return merged;
}; // reducer


exports.mergeDocumentsWithRelationships = mergeDocumentsWithRelationships;

var documents = function documents() {
  var state = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
  var action = arguments.length > 1 ? arguments[1] : undefined;

  if (!(0, _queries.isReceivingData)(action) && !(0, _mutations.isReceivingMutationResult)(action)) {
    return state;
  }

  if (action && action.definition && action.definition.mutationType === _dsl.MutationTypes.DELETE_DOCUMENT) {
    var docId = action.definition.document._id;
    var _type = action.definition.document._type;
    return _objectSpread(_objectSpread({}, state), {}, (0, _defineProperty2.default)({}, _type, (0, _omit.default)(state[_type], docId)));
  }

  var _action$response = action.response,
      data = _action$response.data,
      included = _action$response.included;
  if (!data || Array.isArray(data) && data.length === 0) return state;
  var updatedStateWithIncluded = included ? included.reduce(storeDocument, state) : state;

  if (!Array.isArray(data)) {
    return storeDocument(updatedStateWithIncluded, data);
  }

  return extractAndMergeDocument(data, updatedStateWithIncluded);
};

var _default = documents;
/**
 * Get document from state, by its id
 *
 * @param {import('../types').DocumentsStateSlice} state The documents' slice
 * @param {string} doctype The doctype to target
 * @param {string} id The document id to get
 * @returns {import('../types').CozyClientDocument} the document found by its id
 */

exports.default = _default;

var getDocumentFromSlice = function getDocumentFromSlice() {
  var state = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
  var doctype = arguments.length > 1 ? arguments[1] : undefined;
  var id = arguments.length > 2 ? arguments[2] : undefined;

  if (!doctype) {
    throw new Error('getDocumentFromSlice: Cannot retrieve document with undefined doctype');
  }

  if (!id) {
    throw new Error('getDocumentFromSlice: Cannot retrieve document with undefined id');
  }

  if (!state[doctype]) {
    if (process.env.NODE_ENV !== 'production') {
      _logger.default.info("getDocumentFromSlice: ".concat(doctype, " is absent from the store's documents. State is"), state);
    }

    return null;
  } else if (!state[doctype][id]) {
    if (process.env.NODE_ENV !== 'production') {
      _logger.default.info("getDocumentFromSlice: ".concat(doctype, ":").concat(id, " is absent from the store documents. State is"), state);
    }

    return null;
  }

  return state[doctype][id];
};
/**
 * Get documents from state, by their ids
 *
 * @param {import('../types').DocumentsStateSlice} state The documents' slice
 * @param {string} doctype The doctype to target
 * @param {Array<string>} ids The list of document ids to get
 * @returns {Array<import('../types').CozyClientDocument>} the list of documents found by their ids
 */


exports.getDocumentFromSlice = getDocumentFromSlice;

var getDocumentsFromSlice = function getDocumentsFromSlice() {
  var state = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
  var doctype = arguments.length > 1 ? arguments[1] : undefined;
  var ids = arguments.length > 2 ? arguments[2] : undefined;

  if (!doctype) {
    throw new Error('getDocumentsFromSlice: Cannot retrieve documents with undefined doctype');
  }

  if (!ids || ids.length < 1) {
    throw new Error('getDocumentsFromSlice: Cannot retrieve documents with undefined ids');
  }

  if (!state[doctype]) {
    if (process.env.NODE_ENV !== 'production') {
      _logger.default.info("getDocumentsFromSlice: ".concat(doctype, " is absent from the store's documents. State is"), state);
    }

    return [];
  }

  return ids.map(function (id) {
    return state[doctype][id];
  }).filter(Boolean);
};

exports.getDocumentsFromSlice = getDocumentsFromSlice;

var getCollectionFromSlice = function getCollectionFromSlice() {
  var state = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
  var doctype = arguments.length > 1 ? arguments[1] : undefined;

  if (!doctype) {
    throw new Error('getDocumentFromSlice: Cannot retrieve document with undefined doctype');
  }

  if (!state[doctype]) {
    if (process.env.NODE_ENV !== 'production') {
      _logger.default.info("getCollectionFromSlice: ".concat(doctype, " is absent from the store documents. State is"), state);
    }

    return null;
  }

  return Object.values(state[doctype]);
};
/**
 * 
 * This method has been created in order to get a returned object
   in `data` with the full set on information coming potentially from
  `included`
 
  This method should be somewhere else. The `document` shall not be
  dealt with included / data and so on.
 
  This method takes `data` and `included` and merge both sources
  together. It should be always up-to-date. The returned object
  will be as full of information as it can be.
 
 * @param {Array<import('../types').CozyClientDocument>} newData - Queries documents 
 * @param {import("../types").DocumentsStateSlice} updatedStateWithIncluded documents' state 
 * @returns {import("../types").DocumentsStateSlice} the merged documents state
 */


exports.getCollectionFromSlice = getCollectionFromSlice;

var extractAndMergeDocument = function extractAndMergeDocument(newData, updatedStateWithIncluded) {
  var doctype = newData[0]._type;

  if (!doctype) {
    _logger.default.info('Document without _type', newData[0]);

    throw new Error('Document without _type');
  } // Index docs by id, for faster retrieval


  var newDocsMap = new Map(newData.map(function (doc) {
    return [(0, _helpers.properId)(doc), doc];
  }));
  var mergedState = Object.assign({}, updatedStateWithIncluded); // FIXME: this was introduced for a bug in banks, but it does not look useful

  mergedState[doctype] = Object.assign({}, updatedStateWithIncluded[doctype]);

  if (!mergedState[doctype]) {
    mergedState[doctype] = {};
  }

  var haveDocumentsChanged = false;
  newDocsMap.forEach(function (newDoc, key) {
    var id = key;
    var currentDoc = mergedState[doctype][id];

    if (!currentDoc) {
      mergedState[doctype][id] = newDoc;
      haveDocumentsChanged = true;
    } else if (!(0, _fastDeepEqual.default)(currentDoc, newDoc)) {
      // XXX Here we check if the doc has changed. Note this would be way faster
      // by just relying on revision change, but:
      //  - it does not work if the new doc is partial (i.e. retrieved with a .select query)
      //  - some documents might change without a rev change: typically the thumbnails links
      //    in io.cozy.files, and the io.cozy.settings.instance doc, with additional
      //    fields added by the stack
      //  - some documents don't have any revision
      mergedState[doctype][id] = deepmergeFn(mergedState[doctype][id], newDoc);
      haveDocumentsChanged = true;
    }
  }); // XXX If there is any state change, the updated state should have a new reference,
  // as we heavily rely on referential equality.
  // Conversely, if there is no change, the same reference should be kept

  if (haveDocumentsChanged) {
    return mergedState;
  }

  return updatedStateWithIncluded;
};

exports.extractAndMergeDocument = extractAndMergeDocument;